@@ -14,12 +14,14 @@ struct LxmlDocument {
   xmlDoc (*_c_doc);
   struct __pyx_obj_5etree__BaseParser *_parser;
 };
-__PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlNodeBaseType;
+__PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlElementType;
 
-struct LxmlNodeBase {
+struct LxmlElement {
   PyObject_HEAD
   struct LxmlDocument *_doc;
   xmlNode (*_c_node);
+  PyObject *_tag;
+  PyObject *_attrib;
 };
 __PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlElementTreeType;
 
@@ -27,14 +29,7 @@ struct LxmlElementTree {
   PyObject_HEAD
   struct __pyx_vtabstruct_5etree__ElementTree *__pyx_vtab;
   struct LxmlDocument *_doc;
-  struct LxmlNodeBase *_context_node;
-};
-__PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlElementType;
-
-struct LxmlElement {
-  struct LxmlNodeBase __pyx_base;
-  PyObject *_tag;
-  PyObject *_attrib;
+  struct LxmlElement *_context_node;
 };
 __PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlElementTagMatcherType;
 
@@ -49,7 +44,7 @@ __PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlElementIteratorType;
 
 struct LxmlElementIterator {
   struct LxmlElementTagMatcher __pyx_base;
-  struct LxmlNodeBase *_node;
+  struct LxmlElement *_node;
   xmlNode (*((*_next_element)(xmlNode (*))));
 };
 __PYX_EXTERN_C DL_IMPORT(PyTypeObject) LxmlElementBaseType;
@@ -71,81 +66,81 @@ struct LxmlFallbackElementClassLookup {
   struct LxmlElementClassLookup *fallback;
   PyObject *((*_fallback_function)(PyObject *,struct LxmlDocument *,xmlNode (*)));
 };
-static struct LxmlElement *((*deepcopyNodeToDocument)(struct LxmlDocument *,xmlNode (*)));
-static struct LxmlElementTree *((*elementTreeFactory)(struct LxmlNodeBase *));
-static struct LxmlElementTree *((*newElementTree)(struct LxmlNodeBase *,PyObject *));
-static struct LxmlElement *((*elementFactory)(struct LxmlDocument *,xmlNode (*)));
-static struct LxmlElement *((*makeElement)(PyObject *,struct LxmlDocument *,PyObject *,PyObject *,PyObject *,PyObject *,PyObject *));
-static void ((*setElementClassLookupFunction)(PyObject *((*)(PyObject *,struct LxmlDocument *,xmlNode (*))),PyObject *));
-static PyObject *((*lookupDefaultElementClass)(PyObject *,PyObject *,xmlNode (*)));
-static PyObject *((*lookupNamespaceElementClass)(PyObject *,PyObject *,xmlNode (*)));
-static PyObject *((*callLookupFallback)(struct LxmlFallbackElementClassLookup *,struct LxmlDocument *,xmlNode (*)));
-static int ((*tagMatches)(xmlNode (*),char (*),char (*)));
-static struct LxmlDocument *((*documentOrRaise)(PyObject *));
-static struct LxmlNodeBase *((*rootNodeOrRaise)(PyObject *));
-static PyObject *((*textOf)(xmlNode (*)));
-static PyObject *((*tailOf)(xmlNode (*)));
-static int ((*setNodeText)(xmlNode (*),PyObject *));
-static int ((*setTailText)(xmlNode (*),PyObject *));
+static void ((*appendChild)(struct LxmlElement *,struct LxmlElement *));
 static PyObject *((*attributeValue)(xmlNode (*),xmlAttr (*)));
 static PyObject *((*attributeValueFromNsName)(xmlNode (*),char (*),char (*)));
-static PyObject *((*getAttributeValue)(struct LxmlNodeBase *,PyObject *,PyObject *));
-static PyObject *((*iterattributes)(struct LxmlElement *,int ));
-static int ((*setAttributeValue)(struct LxmlNodeBase *,PyObject *,PyObject *));
-static int ((*delAttribute)(struct LxmlNodeBase *,PyObject *));
+static PyObject *((*callLookupFallback)(struct LxmlFallbackElementClassLookup *,struct LxmlDocument *,xmlNode (*)));
+static struct LxmlElement *((*deepcopyNodeToDocument)(struct LxmlDocument *,xmlNode (*)));
+static int ((*delAttribute)(struct LxmlElement *,PyObject *));
 static int ((*delAttributeFromNsName)(xmlNode (*),char (*),char (*)));
+static struct LxmlDocument *((*documentOrRaise)(PyObject *));
+static struct LxmlElement *((*elementFactory)(struct LxmlDocument *,xmlNode (*)));
+static struct LxmlElementTree *((*elementTreeFactory)(struct LxmlElement *));
 static xmlNode (*((*findChild)(xmlNode (*),Py_ssize_t )));
-static xmlNode (*((*findChildForwards)(xmlNode (*),Py_ssize_t )));
 static xmlNode (*((*findChildBackwards)(xmlNode (*),Py_ssize_t )));
-static xmlNode (*((*nextElement)(xmlNode (*))));
-static xmlNode (*((*previousElement)(xmlNode (*))));
-static void ((*appendChild)(struct LxmlElement *,struct LxmlElement *));
-static PyObject *((*pyunicode)(char (*)));
-static PyObject *((*utf8)(PyObject *));
+static xmlNode (*((*findChildForwards)(xmlNode (*),Py_ssize_t )));
+static xmlNs (*((*findOrBuildNodeNs)(struct LxmlDocument *,xmlNode (*),char (*))));
+static PyObject *((*getAttributeValue)(struct LxmlElement *,PyObject *,PyObject *));
 static PyObject *((*getNsTag)(PyObject *));
+static void ((*initTagMatch)(struct LxmlElementTagMatcher *,PyObject *));
+static void ((*iteratorStoreNext)(struct LxmlElementIterator *,struct LxmlElement *));
+static PyObject *((*iterattributes)(struct LxmlElement *,int ));
+static PyObject *((*lookupDefaultElementClass)(PyObject *,PyObject *,xmlNode (*)));
+static PyObject *((*lookupNamespaceElementClass)(PyObject *,PyObject *,xmlNode (*)));
+static struct LxmlElement *((*makeElement)(PyObject *,struct LxmlDocument *,PyObject *,PyObject *,PyObject *,PyObject *,PyObject *));
 static PyObject *((*namespacedName)(xmlNode (*)));
 static PyObject *((*namespacedNameFromNsName)(char (*),char (*)));
-static void ((*iteratorStoreNext)(struct LxmlElementIterator *,struct LxmlNodeBase *));
-static void ((*initTagMatch)(struct LxmlElementTagMatcher *,PyObject *));
-static xmlNs (*((*findOrBuildNodeNs)(struct LxmlDocument *,xmlNode (*),char (*))));
+static struct LxmlElementTree *((*newElementTree)(struct LxmlElement *,PyObject *));
+static xmlNode (*((*nextElement)(xmlNode (*))));
+static xmlNode (*((*previousElement)(xmlNode (*))));
+static PyObject *((*pyunicode)(char (*)));
+static struct LxmlElement *((*rootNodeOrRaise)(PyObject *));
+static int ((*setAttributeValue)(struct LxmlElement *,PyObject *,PyObject *));
+static void ((*setElementClassLookupFunction)(PyObject *((*)(PyObject *,struct LxmlDocument *,xmlNode (*))),PyObject *));
+static int ((*setNodeText)(xmlNode (*),PyObject *));
+static int ((*setTailText)(xmlNode (*),PyObject *));
+static int ((*tagMatches)(xmlNode (*),char (*),char (*)));
+static PyObject *((*tailOf)(xmlNode (*)));
+static PyObject *((*textOf)(xmlNode (*)));
+static PyObject *((*utf8)(PyObject *));
 static struct {char *s; void **p;} _etree_API[] = {
-  {"deepcopyNodeToDocument", &deepcopyNodeToDocument},
-  {"elementTreeFactory", &elementTreeFactory},
-  {"newElementTree", &newElementTree},
-  {"elementFactory", &elementFactory},
-  {"makeElement", &makeElement},
-  {"setElementClassLookupFunction", &setElementClassLookupFunction},
-  {"lookupDefaultElementClass", &lookupDefaultElementClass},
-  {"lookupNamespaceElementClass", &lookupNamespaceElementClass},
-  {"callLookupFallback", &callLookupFallback},
-  {"tagMatches", &tagMatches},
-  {"documentOrRaise", &documentOrRaise},
-  {"rootNodeOrRaise", &rootNodeOrRaise},
-  {"textOf", &textOf},
-  {"tailOf", &tailOf},
-  {"setNodeText", &setNodeText},
-  {"setTailText", &setTailText},
+  {"appendChild", &appendChild},
   {"attributeValue", &attributeValue},
   {"attributeValueFromNsName", &attributeValueFromNsName},
-  {"getAttributeValue", &getAttributeValue},
-  {"iterattributes", &iterattributes},
-  {"setAttributeValue", &setAttributeValue},
+  {"callLookupFallback", &callLookupFallback},
+  {"deepcopyNodeToDocument", &deepcopyNodeToDocument},
   {"delAttribute", &delAttribute},
   {"delAttributeFromNsName", &delAttributeFromNsName},
+  {"documentOrRaise", &documentOrRaise},
+  {"elementFactory", &elementFactory},
+  {"elementTreeFactory", &elementTreeFactory},
   {"findChild", &findChild},
-  {"findChildForwards", &findChildForwards},
   {"findChildBackwards", &findChildBackwards},
-  {"nextElement", &nextElement},
-  {"previousElement", &previousElement},
-  {"appendChild", &appendChild},
-  {"pyunicode", &pyunicode},
-  {"utf8", &utf8},
+  {"findChildForwards", &findChildForwards},
+  {"findOrBuildNodeNs", &findOrBuildNodeNs},
+  {"getAttributeValue", &getAttributeValue},
   {"getNsTag", &getNsTag},
+  {"initTagMatch", &initTagMatch},
+  {"iteratorStoreNext", &iteratorStoreNext},
+  {"iterattributes", &iterattributes},
+  {"lookupDefaultElementClass", &lookupDefaultElementClass},
+  {"lookupNamespaceElementClass", &lookupNamespaceElementClass},
+  {"makeElement", &makeElement},
   {"namespacedName", &namespacedName},
   {"namespacedNameFromNsName", &namespacedNameFromNsName},
-  {"iteratorStoreNext", &iteratorStoreNext},
-  {"initTagMatch", &initTagMatch},
-  {"findOrBuildNodeNs", &findOrBuildNodeNs},
+  {"newElementTree", &newElementTree},
+  {"nextElement", &nextElement},
+  {"previousElement", &previousElement},
+  {"pyunicode", &pyunicode},
+  {"rootNodeOrRaise", &rootNodeOrRaise},
+  {"setAttributeValue", &setAttributeValue},
+  {"setElementClassLookupFunction", &setElementClassLookupFunction},
+  {"setNodeText", &setNodeText},
+  {"setTailText", &setTailText},
+  {"tagMatches", &tagMatches},
+  {"tailOf", &tailOf},
+  {"textOf", &textOf},
+  {"utf8", &utf8},
   {0, 0}
 };
 
@@ -158,18 +153,25 @@ import_etree(PyObject *module)
                                    module, "_import_c_api");
         if (!c_api_init)
             return -1;
-        if (PyCObject_Check(c_api_init))
+        if (!PyCObject_Check(c_api_init))
         {
-            int (*init)(struct {const char *s; const void **p;}*) =
-                PyCObject_AsVoidPtr(c_api_init);
-            if (!init) {
-                PyErr_SetString(PyExc_RuntimeError,
-                    "module returns NULL pointer for C API call");
-                return -1;
-            }
-            init(_etree_API);
+            Py_DECREF(c_api_init);
+            PyErr_SetString(PyExc_RuntimeError,
+                "etree module provided an invalid C-API reference");
+            return -1;
         }
+
+        int (*init)(struct {const char *s; const void **p;}*) =
+            PyCObject_AsVoidPtr(c_api_init);
         Py_DECREF(c_api_init);
+        if (!init) {
+            PyErr_SetString(PyExc_RuntimeError,
+                "etree module returned NULL pointer for C-API init function");
+            return -1;
+        }
+
+        if (init(_etree_API))
+            return -1;
     }
     return 0;
 }
