@@ -16,6 +16,26 @@
 #endif
 #endif
 
+/* Threading can crash under Python <= 2.4.1 */
+#if PY_VERSION_HEX < 0x02040200
+#ifndef WITHOUT_THREADING
+  #define WITHOUT_THREADING
+#endif
+#endif
+
+#ifdef WITHOUT_THREADING
+  #define PyEval_SaveThread() (NULL)
+  #define PyEval_RestoreThread(state)
+  #define PyGILState_Ensure() (PyGILState_UNLOCKED)
+  #define PyGILState_Release(state)
+#endif
+
+#ifdef WITHOUT_THREADING
+  #define ENABLE_THREADING 0
+#else
+  #define ENABLE_THREADING 1
+#endif
+
 /* libxml2 version specific setup */
 #include "libxml/xmlversion.h"
 #if LIBXML_VERSION < 20621
